# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.rum import rule_create_params, rule_update_params, rule_bulk_create_params
from ..._base_client import make_request_options
from ...types.rum.rum_rule import RUMRule
from ...types.rum.rule_list_response import RuleListResponse
from ...types.rum.rule_delete_response import RuleDeleteResponse
from ...types.rum.rule_bulk_create_response import RuleBulkCreateResponse

__all__ = ["RulesResource", "AsyncRulesResource"]


class RulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesResourceWithStreamingResponse(self)

    def create(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        host: str | Omit = omit,
        inclusive: bool | Omit = omit,
        is_paused: bool | Omit = omit,
        paths: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RUMRule]:
        """
        Creates a new rule in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          inclusive: Whether the rule includes or excludes traffic from being measured.

          is_paused: Whether the rule is paused or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule", account_id=account_id, ruleset_id=ruleset_id
            ),
            body=maybe_transform(
                {
                    "host": host,
                    "inclusive": inclusive,
                    "is_paused": is_paused,
                    "paths": paths,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RUMRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RUMRule]], ResultWrapper[RUMRule]),
        )

    def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        ruleset_id: str,
        host: str | Omit = omit,
        inclusive: bool | Omit = omit,
        is_paused: bool | Omit = omit,
        paths: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RUMRule]:
        """
        Updates a rule in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          rule_id: The Web Analytics rule identifier.

          inclusive: Whether the rule includes or excludes traffic from being measured.

          is_paused: Whether the rule is paused or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule/{rule_id}",
                account_id=account_id,
                ruleset_id=ruleset_id,
                rule_id=rule_id,
            ),
            body=maybe_transform(
                {
                    "host": host,
                    "inclusive": inclusive,
                    "is_paused": is_paused,
                    "paths": paths,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RUMRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RUMRule]], ResultWrapper[RUMRule]),
        )

    def list(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleListResponse]:
        """
        Lists all the rules in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rules", account_id=account_id, ruleset_id=ruleset_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleListResponse]], ResultWrapper[RuleListResponse]),
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        ruleset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """
        Deletes an existing rule from a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          rule_id: The Web Analytics rule identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule/{rule_id}",
                account_id=account_id,
                ruleset_id=ruleset_id,
                rule_id=rule_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    def bulk_create(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        delete_rules: SequenceNotStr[str] | Omit = omit,
        rules: Iterable[rule_bulk_create_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleBulkCreateResponse]:
        """
        Modifies one or more rules in a Web Analytics ruleset with a single request.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          delete_rules: A list of rule identifiers to delete.

          rules: A list of rules to create or update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rules", account_id=account_id, ruleset_id=ruleset_id
            ),
            body=maybe_transform(
                {
                    "delete_rules": delete_rules,
                    "rules": rules,
                },
                rule_bulk_create_params.RuleBulkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleBulkCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleBulkCreateResponse]], ResultWrapper[RuleBulkCreateResponse]),
        )


class AsyncRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        host: str | Omit = omit,
        inclusive: bool | Omit = omit,
        is_paused: bool | Omit = omit,
        paths: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RUMRule]:
        """
        Creates a new rule in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          inclusive: Whether the rule includes or excludes traffic from being measured.

          is_paused: Whether the rule is paused or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule", account_id=account_id, ruleset_id=ruleset_id
            ),
            body=await async_maybe_transform(
                {
                    "host": host,
                    "inclusive": inclusive,
                    "is_paused": is_paused,
                    "paths": paths,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RUMRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RUMRule]], ResultWrapper[RUMRule]),
        )

    async def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        ruleset_id: str,
        host: str | Omit = omit,
        inclusive: bool | Omit = omit,
        is_paused: bool | Omit = omit,
        paths: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RUMRule]:
        """
        Updates a rule in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          rule_id: The Web Analytics rule identifier.

          inclusive: Whether the rule includes or excludes traffic from being measured.

          is_paused: Whether the rule is paused or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule/{rule_id}",
                account_id=account_id,
                ruleset_id=ruleset_id,
                rule_id=rule_id,
            ),
            body=await async_maybe_transform(
                {
                    "host": host,
                    "inclusive": inclusive,
                    "is_paused": is_paused,
                    "paths": paths,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RUMRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RUMRule]], ResultWrapper[RUMRule]),
        )

    async def list(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleListResponse]:
        """
        Lists all the rules in a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rules", account_id=account_id, ruleset_id=ruleset_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleListResponse]], ResultWrapper[RuleListResponse]),
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        ruleset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """
        Deletes an existing rule from a Web Analytics ruleset.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          rule_id: The Web Analytics rule identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rule/{rule_id}",
                account_id=account_id,
                ruleset_id=ruleset_id,
                rule_id=rule_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    async def bulk_create(
        self,
        ruleset_id: str,
        *,
        account_id: str,
        delete_rules: SequenceNotStr[str] | Omit = omit,
        rules: Iterable[rule_bulk_create_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleBulkCreateResponse]:
        """
        Modifies one or more rules in a Web Analytics ruleset with a single request.

        Args:
          account_id: Identifier.

          ruleset_id: The Web Analytics ruleset identifier.

          delete_rules: A list of rule identifiers to delete.

          rules: A list of rules to create or update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/rum/v2/{ruleset_id}/rules", account_id=account_id, ruleset_id=ruleset_id
            ),
            body=await async_maybe_transform(
                {
                    "delete_rules": delete_rules,
                    "rules": rules,
                },
                rule_bulk_create_params.RuleBulkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleBulkCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleBulkCreateResponse]], ResultWrapper[RuleBulkCreateResponse]),
        )


class RulesResourceWithRawResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_raw_response_wrapper(
            rules.create,
        )
        self.update = to_raw_response_wrapper(
            rules.update,
        )
        self.list = to_raw_response_wrapper(
            rules.list,
        )
        self.delete = to_raw_response_wrapper(
            rules.delete,
        )
        self.bulk_create = to_raw_response_wrapper(
            rules.bulk_create,
        )


class AsyncRulesResourceWithRawResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_raw_response_wrapper(
            rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rules.delete,
        )
        self.bulk_create = async_to_raw_response_wrapper(
            rules.bulk_create,
        )


class RulesResourceWithStreamingResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_streamed_response_wrapper(
            rules.create,
        )
        self.update = to_streamed_response_wrapper(
            rules.update,
        )
        self.list = to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            rules.delete,
        )
        self.bulk_create = to_streamed_response_wrapper(
            rules.bulk_create,
        )


class AsyncRulesResourceWithStreamingResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_streamed_response_wrapper(
            rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rules.delete,
        )
        self.bulk_create = async_to_streamed_response_wrapper(
            rules.bulk_create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.devices import network_create_params, network_update_params
from ....types.zero_trust.devices.device_network import DeviceNetwork

__all__ = ["NetworksResource", "AsyncNetworksResource"]


class NetworksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NetworksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        config: network_create_params.Config,
        name: str,
        type: Literal["tls"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Creates a new device managed network.

        Args:
          config: The configuration object containing information for the WARP client to detect
              the managed network.

          name: The name of the device managed network. This name must be unique.

          type: The type of device managed network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/devices/networks", account_id=account_id),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "type": type,
                },
                network_create_params.NetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )

    def update(
        self,
        network_id: str,
        *,
        account_id: str,
        config: network_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        type: Literal["tls"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Updates a configured device managed network.

        Args:
          network_id: API UUID.

          config: The configuration object containing information for the WARP client to detect
              the managed network.

          name: The name of the device managed network. This name must be unique.

          type: The type of device managed network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "type": type,
                },
                network_update_params.NetworkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DeviceNetwork]:
        """
        Fetches a list of managed networks for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/networks", account_id=account_id),
            page=SyncSinglePage[DeviceNetwork],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DeviceNetwork,
        )

    def delete(
        self,
        network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DeviceNetwork]:
        """
        Deletes a device managed network and fetches a list of the remaining device
        managed networks for an account.

        Args:
          network_id: API UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            page=SyncSinglePage[DeviceNetwork],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DeviceNetwork,
            method="delete",
        )

    def get(
        self,
        network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Fetches details for a single managed network.

        Args:
          network_id: API UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )


class AsyncNetworksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNetworksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        config: network_create_params.Config,
        name: str,
        type: Literal["tls"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Creates a new device managed network.

        Args:
          config: The configuration object containing information for the WARP client to detect
              the managed network.

          name: The name of the device managed network. This name must be unique.

          type: The type of device managed network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/devices/networks", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "type": type,
                },
                network_create_params.NetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )

    async def update(
        self,
        network_id: str,
        *,
        account_id: str,
        config: network_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        type: Literal["tls"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Updates a configured device managed network.

        Args:
          network_id: API UUID.

          config: The configuration object containing information for the WARP client to detect
              the managed network.

          name: The name of the device managed network. This name must be unique.

          type: The type of device managed network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "type": type,
                },
                network_update_params.NetworkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DeviceNetwork, AsyncSinglePage[DeviceNetwork]]:
        """
        Fetches a list of managed networks for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/networks", account_id=account_id),
            page=AsyncSinglePage[DeviceNetwork],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DeviceNetwork,
        )

    def delete(
        self,
        network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DeviceNetwork, AsyncSinglePage[DeviceNetwork]]:
        """
        Deletes a device managed network and fetches a list of the remaining device
        managed networks for an account.

        Args:
          network_id: API UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            page=AsyncSinglePage[DeviceNetwork],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DeviceNetwork,
            method="delete",
        )

    async def get(
        self,
        network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceNetwork]:
        """
        Fetches details for a single managed network.

        Args:
          network_id: API UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not network_id:
            raise ValueError(f"Expected a non-empty value for `network_id` but received {network_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/devices/networks/{network_id}", account_id=account_id, network_id=network_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceNetwork]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceNetwork]], ResultWrapper[DeviceNetwork]),
        )


class NetworksResourceWithRawResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = to_raw_response_wrapper(
            networks.create,
        )
        self.update = to_raw_response_wrapper(
            networks.update,
        )
        self.list = to_raw_response_wrapper(
            networks.list,
        )
        self.delete = to_raw_response_wrapper(
            networks.delete,
        )
        self.get = to_raw_response_wrapper(
            networks.get,
        )


class AsyncNetworksResourceWithRawResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = async_to_raw_response_wrapper(
            networks.create,
        )
        self.update = async_to_raw_response_wrapper(
            networks.update,
        )
        self.list = async_to_raw_response_wrapper(
            networks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            networks.delete,
        )
        self.get = async_to_raw_response_wrapper(
            networks.get,
        )


class NetworksResourceWithStreamingResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = to_streamed_response_wrapper(
            networks.create,
        )
        self.update = to_streamed_response_wrapper(
            networks.update,
        )
        self.list = to_streamed_response_wrapper(
            networks.list,
        )
        self.delete = to_streamed_response_wrapper(
            networks.delete,
        )
        self.get = to_streamed_response_wrapper(
            networks.get,
        )


class AsyncNetworksResourceWithStreamingResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = async_to_streamed_response_wrapper(
            networks.create,
        )
        self.update = async_to_streamed_response_wrapper(
            networks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            networks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            networks.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            networks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ..._models import BaseModel

__all__ = ["EvaluationTypeListResponse"]


class EvaluationTypeListResponse(BaseModel):
    id: str

    created_at: datetime

    description: str

    enable: bool

    mandatory: bool

    modified_at: datetime

    name: str

    type: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TieredCachingGetResponse"]


class TieredCachingGetResponse(BaseModel):
    id: Literal["tiered_caching"]
    """The identifier of the caching setting."""

    editable: bool
    """Whether the setting is editable."""

    value: Literal["on", "off"]
    """Value of the Tiered Cache zone setting."""

    modified_on: Optional[datetime] = None
    """Last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .summary_bot_class_params import SummaryBotClassParams as SummaryBotClassParams
from .summary_bot_class_response import SummaryBotClassResponse as SummaryBotClassResponse
from .summary_compromised_params import SummaryCompromisedParams as SummaryCompromisedParams
from .summary_compromised_response import SummaryCompromisedResponse as SummaryCompromisedResponse
from .timeseries_group_bot_class_params import TimeseriesGroupBotClassParams as TimeseriesGroupBotClassParams
from .timeseries_group_bot_class_response import TimeseriesGroupBotClassResponse as TimeseriesGroupBotClassResponse
from .timeseries_group_compromised_params import TimeseriesGroupCompromisedParams as TimeseriesGroupCompromisedParams
from .timeseries_group_compromised_response import (
    TimeseriesGroupCompromisedResponse as TimeseriesGroupCompromisedResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PatternParam"]


class PatternParam(TypedDict, total=False):
    regex: Required[str]

    validation: Literal["luhn"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MTLSCertificateCreateParams"]


class MTLSCertificateCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    ca: Required[bool]
    """Indicates whether the certificate is a CA or leaf certificate."""

    certificates: Required[str]
    """The uploaded root CA certificate."""

    name: str
    """Optional unique name for the certificate. Only used for human readability."""

    private_key: str
    """The private key for the certificate.

    This field is only needed for specific use cases such as using a custom
    certificate with Zero Trust's block page.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .prioritize import (
    PrioritizeResource,
    AsyncPrioritizeResource,
    PrioritizeResourceWithRawResponse,
    AsyncPrioritizeResourceWithRawResponse,
    PrioritizeResourceWithStreamingResponse,
    AsyncPrioritizeResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.custom_hostnames import BundleMethod
from ...types.custom_certificates import (
    custom_certificate_edit_params,
    custom_certificate_list_params,
    custom_certificate_create_params,
)
from ...types.custom_hostnames.bundle_method import BundleMethod
from ...types.custom_certificates.custom_certificate import CustomCertificate
from ...types.custom_certificates.geo_restrictions_param import GeoRestrictionsParam
from ...types.custom_certificates.custom_certificate_delete_response import CustomCertificateDeleteResponse

__all__ = ["CustomCertificatesResource", "AsyncCustomCertificatesResource"]


class CustomCertificatesResource(SyncAPIResource):
    @cached_property
    def prioritize(self) -> PrioritizeResource:
        return PrioritizeResource(self._client)

    @cached_property
    def with_raw_response(self) -> CustomCertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomCertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomCertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomCertificatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        certificate: str,
        private_key: str,
        bundle_method: BundleMethod | Omit = omit,
        custom_csr_id: str | Omit = omit,
        deploy: Literal["staging", "production"] | Omit = omit,
        geo_restrictions: GeoRestrictionsParam | Omit = omit,
        policy: str | Omit = omit,
        type: Literal["legacy_custom", "sni_custom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """
        Upload a new SSL certificate for a zone.

        Args:
          zone_id: Identifier.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          private_key: The zone's private key.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          custom_csr_id: The identifier for the Custom CSR that was used.

          deploy: The environment to deploy the certificate to, defaults to production

          geo_restrictions: Specify the region where your private key can be held locally for optimal TLS
              performance. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Options allow distribution to
              only to U.S. data centers, only to E.U. data centers, or only to highest
              security data centers. Default distribution is to all Cloudflare datacenters,
              for optimal performance.

          policy: Specify the policy that determines the region where your private key will be
              held locally. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Any combination of countries,
              specified by their two letter country code
              (https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements)
              can be chosen, such as 'country: IN', as well as 'region: EU' which refers to
              the EU region. If there are too few data centers satisfying the policy, it will
              be rejected. Note: The API accepts this field as either "policy" or
              "policy_restrictions" in requests. Responses return this field as
              "policy_restrictions".

          type: The type 'legacy_custom' enables support for legacy clients which do not include
              SNI in the TLS handshake.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/custom_certificates", zone_id=zone_id),
            body=maybe_transform(
                {
                    "certificate": certificate,
                    "private_key": private_key,
                    "bundle_method": bundle_method,
                    "custom_csr_id": custom_csr_id,
                    "deploy": deploy,
                    "geo_restrictions": geo_restrictions,
                    "policy": policy,
                    "type": type,
                },
                custom_certificate_create_params.CustomCertificateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )

    def list(
        self,
        *,
        zone_id: str,
        match: Literal["any", "all"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["active", "expired", "deleted", "pending", "initializing"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[CustomCertificate]:
        """List, search, and filter all of your custom SSL certificates.

        The higher
        priority will break ties across overlapping 'legacy_custom' certificates, but
        'legacy_custom' certificates will always supercede 'sni_custom' certificates.

        Args:
          zone_id: Identifier.

          match: Whether to match all search requirements or at least one (any).

          page: Page number of paginated results.

          per_page: Number of zones per page.

          status: Status of the zone's custom SSL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/custom_certificates", zone_id=zone_id),
            page=SyncV4PagePaginationArray[CustomCertificate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "match": match,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    custom_certificate_list_params.CustomCertificateListParams,
                ),
            ),
            model=CustomCertificate,
        )

    def delete(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificateDeleteResponse]:
        """
        Remove a SSL certificate from a zone.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return self._delete(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[CustomCertificateDeleteResponse]], ResultWrapper[CustomCertificateDeleteResponse]
            ),
        )

    def edit(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        bundle_method: BundleMethod | Omit = omit,
        certificate: str | Omit = omit,
        custom_csr_id: str | Omit = omit,
        deploy: Literal["staging", "production"] | Omit = omit,
        geo_restrictions: GeoRestrictionsParam | Omit = omit,
        policy: str | Omit = omit,
        private_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """Upload a new private key and/or PEM/CRT for the SSL certificate.

        Note: PATCHing
        a configuration for sni_custom certificates will result in a new resource id
        being returned, and the previous one being deleted.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          custom_csr_id: The identifier for the Custom CSR that was used.

          deploy: The environment to deploy the certificate to, defaults to production

          geo_restrictions: Specify the region where your private key can be held locally for optimal TLS
              performance. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Options allow distribution to
              only to U.S. data centers, only to E.U. data centers, or only to highest
              security data centers. Default distribution is to all Cloudflare datacenters,
              for optimal performance.

          policy: Specify the policy that determines the region where your private key will be
              held locally. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Any combination of countries,
              specified by their two letter country code
              (https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements)
              can be chosen, such as 'country: IN', as well as 'region: EU' which refers to
              the EU region. If there are too few data centers satisfying the policy, it will
              be rejected. Note: The API accepts this field as either "policy" or
              "policy_restrictions" in requests. Responses return this field as
              "policy_restrictions".

          private_key: The zone's private key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return self._patch(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            body=maybe_transform(
                {
                    "bundle_method": bundle_method,
                    "certificate": certificate,
                    "custom_csr_id": custom_csr_id,
                    "deploy": deploy,
                    "geo_restrictions": geo_restrictions,
                    "policy": policy,
                    "private_key": private_key,
                },
                custom_certificate_edit_params.CustomCertificateEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )

    def get(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """
        Retrieves details for a specific custom SSL certificate, including certificate
        metadata, bundle method, geographic restrictions, and associated keyless server
        configuration.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return self._get(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )


class AsyncCustomCertificatesResource(AsyncAPIResource):
    @cached_property
    def prioritize(self) -> AsyncPrioritizeResource:
        return AsyncPrioritizeResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCustomCertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomCertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomCertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomCertificatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        certificate: str,
        private_key: str,
        bundle_method: BundleMethod | Omit = omit,
        custom_csr_id: str | Omit = omit,
        deploy: Literal["staging", "production"] | Omit = omit,
        geo_restrictions: GeoRestrictionsParam | Omit = omit,
        policy: str | Omit = omit,
        type: Literal["legacy_custom", "sni_custom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """
        Upload a new SSL certificate for a zone.

        Args:
          zone_id: Identifier.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          private_key: The zone's private key.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          custom_csr_id: The identifier for the Custom CSR that was used.

          deploy: The environment to deploy the certificate to, defaults to production

          geo_restrictions: Specify the region where your private key can be held locally for optimal TLS
              performance. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Options allow distribution to
              only to U.S. data centers, only to E.U. data centers, or only to highest
              security data centers. Default distribution is to all Cloudflare datacenters,
              for optimal performance.

          policy: Specify the policy that determines the region where your private key will be
              held locally. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Any combination of countries,
              specified by their two letter country code
              (https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements)
              can be chosen, such as 'country: IN', as well as 'region: EU' which refers to
              the EU region. If there are too few data centers satisfying the policy, it will
              be rejected. Note: The API accepts this field as either "policy" or
              "policy_restrictions" in requests. Responses return this field as
              "policy_restrictions".

          type: The type 'legacy_custom' enables support for legacy clients which do not include
              SNI in the TLS handshake.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/custom_certificates", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "certificate": certificate,
                    "private_key": private_key,
                    "bundle_method": bundle_method,
                    "custom_csr_id": custom_csr_id,
                    "deploy": deploy,
                    "geo_restrictions": geo_restrictions,
                    "policy": policy,
                    "type": type,
                },
                custom_certificate_create_params.CustomCertificateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )

    def list(
        self,
        *,
        zone_id: str,
        match: Literal["any", "all"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["active", "expired", "deleted", "pending", "initializing"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomCertificate, AsyncV4PagePaginationArray[CustomCertificate]]:
        """List, search, and filter all of your custom SSL certificates.

        The higher
        priority will break ties across overlapping 'legacy_custom' certificates, but
        'legacy_custom' certificates will always supercede 'sni_custom' certificates.

        Args:
          zone_id: Identifier.

          match: Whether to match all search requirements or at least one (any).

          page: Page number of paginated results.

          per_page: Number of zones per page.

          status: Status of the zone's custom SSL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/custom_certificates", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[CustomCertificate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "match": match,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    custom_certificate_list_params.CustomCertificateListParams,
                ),
            ),
            model=CustomCertificate,
        )

    async def delete(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificateDeleteResponse]:
        """
        Remove a SSL certificate from a zone.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return await self._delete(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[CustomCertificateDeleteResponse]], ResultWrapper[CustomCertificateDeleteResponse]
            ),
        )

    async def edit(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        bundle_method: BundleMethod | Omit = omit,
        certificate: str | Omit = omit,
        custom_csr_id: str | Omit = omit,
        deploy: Literal["staging", "production"] | Omit = omit,
        geo_restrictions: GeoRestrictionsParam | Omit = omit,
        policy: str | Omit = omit,
        private_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """Upload a new private key and/or PEM/CRT for the SSL certificate.

        Note: PATCHing
        a configuration for sni_custom certificates will result in a new resource id
        being returned, and the previous one being deleted.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          custom_csr_id: The identifier for the Custom CSR that was used.

          deploy: The environment to deploy the certificate to, defaults to production

          geo_restrictions: Specify the region where your private key can be held locally for optimal TLS
              performance. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Options allow distribution to
              only to U.S. data centers, only to E.U. data centers, or only to highest
              security data centers. Default distribution is to all Cloudflare datacenters,
              for optimal performance.

          policy: Specify the policy that determines the region where your private key will be
              held locally. HTTPS connections to any excluded data center will still be fully
              encrypted, but will incur some latency while Keyless SSL is used to complete the
              handshake with the nearest allowed data center. Any combination of countries,
              specified by their two letter country code
              (https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements)
              can be chosen, such as 'country: IN', as well as 'region: EU' which refers to
              the EU region. If there are too few data centers satisfying the policy, it will
              be rejected. Note: The API accepts this field as either "policy" or
              "policy_restrictions" in requests. Responses return this field as
              "policy_restrictions".

          private_key: The zone's private key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return await self._patch(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            body=await async_maybe_transform(
                {
                    "bundle_method": bundle_method,
                    "certificate": certificate,
                    "custom_csr_id": custom_csr_id,
                    "deploy": deploy,
                    "geo_restrictions": geo_restrictions,
                    "policy": policy,
                    "private_key": private_key,
                },
                custom_certificate_edit_params.CustomCertificateEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )

    async def get(
        self,
        custom_certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCertificate]:
        """
        Retrieves details for a specific custom SSL certificate, including certificate
        metadata, bundle method, geographic restrictions, and associated keyless server
        configuration.

        Args:
          zone_id: Identifier.

          custom_certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_certificate_id:
            raise ValueError(
                f"Expected a non-empty value for `custom_certificate_id` but received {custom_certificate_id!r}"
            )
        return await self._get(
            path_template(
                "/zones/{zone_id}/custom_certificates/{custom_certificate_id}",
                zone_id=zone_id,
                custom_certificate_id=custom_certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCertificate]], ResultWrapper[CustomCertificate]),
        )


class CustomCertificatesResourceWithRawResponse:
    def __init__(self, custom_certificates: CustomCertificatesResource) -> None:
        self._custom_certificates = custom_certificates

        self.create = to_raw_response_wrapper(
            custom_certificates.create,
        )
        self.list = to_raw_response_wrapper(
            custom_certificates.list,
        )
        self.delete = to_raw_response_wrapper(
            custom_certificates.delete,
        )
        self.edit = to_raw_response_wrapper(
            custom_certificates.edit,
        )
        self.get = to_raw_response_wrapper(
            custom_certificates.get,
        )

    @cached_property
    def prioritize(self) -> PrioritizeResourceWithRawResponse:
        return PrioritizeResourceWithRawResponse(self._custom_certificates.prioritize)


class AsyncCustomCertificatesResourceWithRawResponse:
    def __init__(self, custom_certificates: AsyncCustomCertificatesResource) -> None:
        self._custom_certificates = custom_certificates

        self.create = async_to_raw_response_wrapper(
            custom_certificates.create,
        )
        self.list = async_to_raw_response_wrapper(
            custom_certificates.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_certificates.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            custom_certificates.edit,
        )
        self.get = async_to_raw_response_wrapper(
            custom_certificates.get,
        )

    @cached_property
    def prioritize(self) -> AsyncPrioritizeResourceWithRawResponse:
        return AsyncPrioritizeResourceWithRawResponse(self._custom_certificates.prioritize)


class CustomCertificatesResourceWithStreamingResponse:
    def __init__(self, custom_certificates: CustomCertificatesResource) -> None:
        self._custom_certificates = custom_certificates

        self.create = to_streamed_response_wrapper(
            custom_certificates.create,
        )
        self.list = to_streamed_response_wrapper(
            custom_certificates.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom_certificates.delete,
        )
        self.edit = to_streamed_response_wrapper(
            custom_certificates.edit,
        )
        self.get = to_streamed_response_wrapper(
            custom_certificates.get,
        )

    @cached_property
    def prioritize(self) -> PrioritizeResourceWithStreamingResponse:
        return PrioritizeResourceWithStreamingResponse(self._custom_certificates.prioritize)


class AsyncCustomCertificatesResourceWithStreamingResponse:
    def __init__(self, custom_certificates: AsyncCustomCertificatesResource) -> None:
        self._custom_certificates = custom_certificates

        self.create = async_to_streamed_response_wrapper(
            custom_certificates.create,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_certificates.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_certificates.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            custom_certificates.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_certificates.get,
        )

    @cached_property
    def prioritize(self) -> AsyncPrioritizeResourceWithStreamingResponse:
        return AsyncPrioritizeResourceWithStreamingResponse(self._custom_certificates.prioritize)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ItemGetResponse"]


class ItemGetResponse(BaseModel):
    id: str

    checksum: str

    chunks_count: Optional[int] = None

    created_at: datetime

    file_size: Optional[float] = None

    key: str

    last_seen_at: datetime

    namespace: str

    next_action: Optional[Literal["INDEX", "DELETE"]] = None

    source_id: Optional[str] = None
    """Identifies which data source this item belongs to.

    "builtin" for uploaded files, "{type}:{source}" for external sources, null for
    legacy items.
    """

    status: Literal["queued", "running", "completed", "error", "skipped", "outdated"]

    error: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["RUMRule"]


class RUMRule(BaseModel):
    id: Optional[str] = None
    """The Web Analytics rule identifier."""

    created: Optional[datetime] = None

    host: Optional[str] = None
    """The hostname the rule will be applied to."""

    inclusive: Optional[bool] = None
    """Whether the rule includes or excludes traffic from being measured."""

    is_paused: Optional[bool] = None
    """Whether the rule is paused or not."""

    paths: Optional[List[str]] = None
    """The paths the rule will be applied to."""

    priority: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .domain import (
    DomainResource,
    AsyncDomainResource,
    DomainResourceWithRawResponse,
    AsyncDomainResourceWithRawResponse,
    DomainResourceWithStreamingResponse,
    AsyncDomainResourceWithStreamingResponse,
)
from .ranking import (
    RankingResource,
    AsyncRankingResource,
    RankingResourceWithRawResponse,
    AsyncRankingResourceWithRawResponse,
    RankingResourceWithStreamingResponse,
    AsyncRankingResourceWithStreamingResponse,
)
from .internet_services import (
    InternetServicesResource,
    AsyncInternetServicesResource,
    InternetServicesResourceWithRawResponse,
    AsyncInternetServicesResourceWithRawResponse,
    InternetServicesResourceWithStreamingResponse,
    AsyncInternetServicesResourceWithStreamingResponse,
)

__all__ = [
    "DomainResource",
    "AsyncDomainResource",
    "DomainResourceWithRawResponse",
    "AsyncDomainResourceWithRawResponse",
    "DomainResourceWithStreamingResponse",
    "AsyncDomainResourceWithStreamingResponse",
    "InternetServicesResource",
    "AsyncInternetServicesResource",
    "InternetServicesResourceWithRawResponse",
    "AsyncInternetServicesResourceWithRawResponse",
    "InternetServicesResourceWithStreamingResponse",
    "AsyncInternetServicesResourceWithStreamingResponse",
    "RankingResource",
    "AsyncRankingResource",
    "RankingResourceWithRawResponse",
    "AsyncRankingResourceWithRawResponse",
    "RankingResourceWithStreamingResponse",
    "AsyncRankingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel

__all__ = ["SettingEditResponse"]


class SettingEditResponse(BaseModel):
    allow_iframe: Optional[bool] = None
    """Enables loading application content in an iFrame."""

    skip_interstitial: Optional[bool] = None
    """Enables automatic authentication through cloudflared."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .subscription_configuration import SubscriptionConfiguration as SubscriptionConfiguration
from .bot_management_get_response import BotManagementGetResponse as BotManagementGetResponse
from .bot_fight_mode_configuration import BotFightModeConfiguration as BotFightModeConfiguration
from .bot_management_update_params import BotManagementUpdateParams as BotManagementUpdateParams
from .bot_management_update_response import BotManagementUpdateResponse as BotManagementUpdateResponse
from .subscription_configuration_param import SubscriptionConfigurationParam as SubscriptionConfigurationParam
from .bot_fight_mode_configuration_param import BotFightModeConfigurationParam as BotFightModeConfigurationParam
from .super_bot_fight_mode_likely_configuration import (
    SuperBotFightModeLikelyConfiguration as SuperBotFightModeLikelyConfiguration,
)
from .super_bot_fight_mode_definitely_configuration import (
    SuperBotFightModeDefinitelyConfiguration as SuperBotFightModeDefinitelyConfiguration,
)
from .super_bot_fight_mode_likely_configuration_param import (
    SuperBotFightModeLikelyConfigurationParam as SuperBotFightModeLikelyConfigurationParam,
)
from .super_bot_fight_mode_definitely_configuration_param import (
    SuperBotFightModeDefinitelyConfigurationParam as SuperBotFightModeDefinitelyConfigurationParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["DeviceMatchParam"]


class DeviceMatchParam(TypedDict, total=False):
    platform: Literal["windows", "mac", "linux", "android", "ios", "chromeos"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BlockSenderEditParams"]


class BlockSenderEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    comments: Optional[str]

    is_regex: bool

    pattern: str

    pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"]
    """
    Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
    creating or updating policies, but may be returned for existing entries.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["IPListParams"]


class IPListParams(TypedDict, total=False):
    networks: str
    """Specified as `jdcloud` to list IPs used by JD Cloud data centers."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .logging_param import LoggingParam

__all__ = [
    "RewriteRuleParam",
    "ActionParameters",
    "ActionParametersHeaders",
    "ActionParametersHeadersAddStaticHeader",
    "ActionParametersHeadersAddDynamicHeader",
    "ActionParametersHeadersSetStaticHeader",
    "ActionParametersHeadersSetDynamicHeader",
    "ActionParametersHeadersRemoveHeader",
    "ActionParametersURI",
    "ActionParametersURIURIPath",
    "ActionParametersURIURIPathPath",
    "ActionParametersURIURIQuery",
    "ActionParametersURIURIQueryQuery",
    "ExposedCredentialCheck",
    "Ratelimit",
]


class ActionParametersHeadersAddStaticHeader(TypedDict, total=False):
    """A header with a static value to add."""

    operation: Required[Literal["add"]]
    """The operation to perform on the header."""

    value: Required[str]
    """A static value for the header."""


class ActionParametersHeadersAddDynamicHeader(TypedDict, total=False):
    """A header with a dynamic value to add."""

    expression: Required[str]
    """An expression that evaluates to a value for the header."""

    operation: Required[Literal["add"]]
    """The operation to perform on the header."""


class ActionParametersHeadersSetStaticHeader(TypedDict, total=False):
    """A header with a static value to set."""

    operation: Required[Literal["set"]]
    """The operation to perform on the header."""

    value: Required[str]
    """A static value for the header."""


class ActionParametersHeadersSetDynamicHeader(TypedDict, total=False):
    """A header with a dynamic value to set."""

    expression: Required[str]
    """An expression that evaluates to a value for the header."""

    operation: Required[Literal["set"]]
    """The operation to perform on the header."""


class ActionParametersHeadersRemoveHeader(TypedDict, total=False):
    """A header to remove."""

    operation: Required[Literal["remove"]]
    """The operation to perform on the header."""


ActionParametersHeaders: TypeAlias = Union[
    ActionParametersHeadersAddStaticHeader,
    ActionParametersHeadersAddDynamicHeader,
    ActionParametersHeadersSetStaticHeader,
    ActionParametersHeadersSetDynamicHeader,
    ActionParametersHeadersRemoveHeader,
]


class ActionParametersURIURIPathPath(TypedDict, total=False):
    """A URI path rewrite."""

    expression: str
    """An expression that evaluates to a value to rewrite the URI path to."""

    value: str
    """A value to rewrite the URI path to."""


class ActionParametersURIURIPath(TypedDict, total=False):
    """A URI path rewrite."""

    path: Required[ActionParametersURIURIPathPath]
    """A URI path rewrite."""


class ActionParametersURIURIQueryQuery(TypedDict, total=False):
    """A URI query rewrite."""

    expression: str
    """An expression that evaluates to a value to rewrite the URI query to."""

    value: str
    """A value to rewrite the URI query to."""


class ActionParametersURIURIQuery(TypedDict, total=False):
    """A URI query rewrite."""

    query: Required[ActionParametersURIURIQueryQuery]
    """A URI query rewrite."""


ActionParametersURI: TypeAlias = Union[ActionParametersURIURIPath, ActionParametersURIURIQuery]


class ActionParameters(TypedDict, total=False):
    """The parameters configuring the rule's action."""

    headers: Dict[str, ActionParametersHeaders]
    """A map of headers to rewrite."""

    uri: ActionParametersURI
    """A URI path rewrite."""


class ExposedCredentialCheck(TypedDict, total=False):
    """Configuration for exposed credential checking."""

    password_expression: Required[str]
    """An expression that selects the password used in the credentials check."""

    username_expression: Required[str]
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(TypedDict, total=False):
    """An object configuring the rule's rate limit behavior."""

    characteristics: Required[SequenceNotStr[str]]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: Required[int]
    """Period in seconds over which the counter is being incremented."""

    counting_expression: str
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: int
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: int
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: bool
    """Whether counting is only performed when an origin is reached."""

    score_per_period: int
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: str
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class RewriteRuleParam(TypedDict, total=False):
    id: str
    """The unique ID of the rule."""

    action: Literal["rewrite"]
    """The action to perform when the rule matches."""

    action_parameters: ActionParameters
    """The parameters configuring the rule's action."""

    description: str
    """An informative description of the rule."""

    enabled: bool
    """Whether the rule should be executed."""

    exposed_credential_check: ExposedCredentialCheck
    """Configuration for exposed credential checking."""

    expression: str
    """The expression defining which traffic will match the rule."""

    logging: LoggingParam
    """An object configuring the rule's logging behavior."""

    ratelimit: Ratelimit
    """An object configuring the rule's rate limit behavior."""

    ref: str
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["JobGetResponse"]


class JobGetResponse(BaseModel):
    id: str

    source: Literal["user", "schedule"]

    description: Optional[str] = None

    end_reason: Optional[str] = None

    ended_at: Optional[str] = None

    last_seen_at: Optional[str] = None

    started_at: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Event"]


class Event(BaseModel):
    id: Optional[str] = None

    created_on: Optional[datetime] = None

    custom_page_html: Optional[str] = None
    """
    If set, the event will override the waiting room's `custom_page_html` property
    while it is active. If null, the event will inherit it.
    """

    description: Optional[str] = None
    """A note that you can use to add more details about the event."""

    disable_session_renewal: Optional[bool] = None
    """
    If set, the event will override the waiting room's `disable_session_renewal`
    property while it is active. If null, the event will inherit it.
    """

    event_end_time: Optional[str] = None
    """An ISO 8601 timestamp that marks the end of the event."""

    event_start_time: Optional[str] = None
    """An ISO 8601 timestamp that marks the start of the event.

    At this time, queued users will be processed with the event's configuration. The
    start time must be at least one minute before `event_end_time`.
    """

    modified_on: Optional[datetime] = None

    name: Optional[str] = None
    """A unique name to identify the event.

    Only alphanumeric characters, hyphens and underscores are allowed.
    """

    new_users_per_minute: Optional[int] = None
    """
    If set, the event will override the waiting room's `new_users_per_minute`
    property while it is active. If null, the event will inherit it. This can only
    be set if the event's `total_active_users` property is also set.
    """

    prequeue_start_time: Optional[str] = None
    """
    An ISO 8601 timestamp that marks when to begin queueing all users before the
    event starts. The prequeue must start at least five minutes before
    `event_start_time`.
    """

    queueing_method: Optional[str] = None
    """
    If set, the event will override the waiting room's `queueing_method` property
    while it is active. If null, the event will inherit it.
    """

    session_duration: Optional[int] = None
    """
    If set, the event will override the waiting room's `session_duration` property
    while it is active. If null, the event will inherit it.
    """

    shuffle_at_event_start: Optional[bool] = None
    """
    If enabled, users in the prequeue will be shuffled randomly at the
    `event_start_time`. Requires that `prequeue_start_time` is not null. This is
    useful for situations when many users will join the event prequeue at the same
    time and you want to shuffle them to ensure fairness. Naturally, it makes the
    most sense to enable this feature when the `queueing_method` during the event
    respects ordering such as **fifo**, or else the shuffling may be unnecessary.
    """

    suspended: Optional[bool] = None
    """Suspends or allows an event.

    If set to `true`, the event is ignored and traffic will be handled based on the
    waiting room configuration.
    """

    total_active_users: Optional[int] = None
    """
    If set, the event will override the waiting room's `total_active_users` property
    while it is active. If null, the event will inherit it. This can only be set if
    the event's `new_users_per_minute` property is also set.
    """

    turnstile_action: Optional[Literal["log", "infinite_queue"]] = None
    """
    If set, the event will override the waiting room's `turnstile_action` property
    while it is active. If null, the event will inherit it.
    """

    turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]] = None
    """
    If set, the event will override the waiting room's `turnstile_mode` property
    while it is active. If null, the event will inherit it.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .test_stat_over_time import TestStatOverTime as TestStatOverTime
from .percentile_get_params import PercentileGetParams as PercentileGetParams
from .http_details_percentiles import HTTPDetailsPercentiles as HTTPDetailsPercentiles



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TCPConfiguration"]


class TCPConfiguration(BaseModel):
    """Parameters specific to TCP health check."""

    method: Optional[Literal["connection_established"]] = None
    """The TCP connection method to use for the health check."""

    port: Optional[int] = None
    """Port number to connect to for the health check. Defaults to 80."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Status"]

Status: TypeAlias = Literal["active", "pending_reactivation", "pending_revocation", "revoked"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from .label import Label
from ...._models import BaseModel

__all__ = ["Priority"]


class Priority(BaseModel):
    id: str
    """UUID."""

    created: datetime
    """Priority creation time."""

    labels: List[Label]
    """List of labels."""

    priority: int
    """Priority."""

    requirement: str
    """Requirement."""

    tlp: Literal["clear", "amber", "amber-strict", "green", "red"]
    """The CISA defined Traffic Light Protocol (TLP)."""

    updated: datetime
    """Priority last updated time."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["IPV6ConfigurationParam"]


class IPV6ConfigurationParam(TypedDict, total=False):
    target: Literal["ip6"]
    """The configuration target.

    You must set the target to `ip6` when specifying an IPv6 address in the rule.
    """

    value: str
    """The IPv6 address to match."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ReverseDNSEditParams"]


class ReverseDNSEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    ptr: Dict[str, str]
    """Map of cluster IP addresses to PTR record contents"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["DatasetCreateParams"]


class DatasetCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    is_public: Required[Annotated[bool, PropertyInfo(alias="isPublic")]]
    """If true, then anyone can search the dataset.

    If false, then its limited to the account.
    """

    name: Required[str]
    """Used to describe the dataset within the account context."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["TargetCloseResponse"]


class TargetCloseResponse(BaseModel):
    message: str
    """Target is closing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["AppPostResponse", "Data", "DataApp"]


class DataApp(BaseModel):
    id: Optional[str] = None

    created_at: Optional[str] = None

    name: Optional[str] = None


class Data(BaseModel):
    app: Optional[DataApp] = None


class AppPostResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ....._models import BaseModel

__all__ = ["ActiveSessionListResponse", "Metadata", "MetadataApps"]


class MetadataApps(BaseModel):
    hostname: Optional[str] = None

    name: Optional[str] = None

    type: Optional[str] = None

    uid: Optional[str] = None


class Metadata(BaseModel):
    apps: Optional[Dict[str, MetadataApps]] = None

    expires: Optional[int] = None

    iat: Optional[int] = None

    nonce: Optional[str] = None

    ttl: Optional[int] = None


class ActiveSessionListResponse(BaseModel):
    expiration: Optional[int] = None

    metadata: Optional[Metadata] = None

    name: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["SCIMConfigAuthenticationHTTPBasic"]


class SCIMConfigAuthenticationHTTPBasic(BaseModel):
    """
    Attributes for configuring HTTP Basic authentication scheme for SCIM provisioning to an application.
    """

    password: str
    """Password used to authenticate with the remote SCIM service."""

    scheme: Literal["httpbasic"]
    """The authentication scheme to use when making SCIM requests to this application."""

    user: str
    """User name used to authenticate with the remote SCIM service."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..decision import Decision
from ....._models import BaseModel
from .access_rule import AccessRule
from ..approval_group import ApprovalGroup

__all__ = ["PolicyCreateResponse", "ConnectionRules", "ConnectionRulesRDP", "MfaConfig"]


class ConnectionRulesRDP(BaseModel):
    """The RDP-specific rules that define clipboard behavior for RDP connections."""

    allowed_clipboard_local_to_remote_formats: Optional[List[Literal["text"]]] = None
    """
    Clipboard formats allowed when copying from local machine to remote RDP session.
    """

    allowed_clipboard_remote_to_local_formats: Optional[List[Literal["text"]]] = None
    """
    Clipboard formats allowed when copying from remote RDP session to local machine.
    """


class ConnectionRules(BaseModel):
    """
    The rules that define how users may connect to targets secured by your application.
    """

    rdp: Optional[ConnectionRulesRDP] = None
    """The RDP-specific rules that define clipboard behavior for RDP connections."""


class MfaConfig(BaseModel):
    """Configures multi-factor authentication (MFA) settings."""

    allowed_authenticators: Optional[List[Literal["totp", "biometrics", "security_key"]]] = None
    """Lists the MFA methods that users can authenticate with."""

    mfa_disabled: Optional[bool] = None
    """Indicates whether to disable MFA for this resource.

    This option is available at the application and policy level.
    """

    session_duration: Optional[str] = None
    """Defines the duration of an MFA session.

    Must be in minutes (m) or hours (h). Minimum: 0m. Maximum: 720h (30 days).
    Examples:`5m` or `24h`.
    """


class PolicyCreateResponse(BaseModel):
    id: Optional[str] = None
    """The UUID of the policy"""

    approval_groups: Optional[List[ApprovalGroup]] = None
    """Administrators who can approve a temporary authentication request."""

    approval_required: Optional[bool] = None
    """
    Requires the user to request access from an administrator at the start of each
    session.
    """

    connection_rules: Optional[ConnectionRules] = None
    """
    The rules that define how users may connect to targets secured by your
    application.
    """

    created_at: Optional[datetime] = None

    decision: Optional[Decision] = None
    """The action Access will take if a user matches this policy.

    Infrastructure application policies can only use the Allow action.
    """

    exclude: Optional[List[AccessRule]] = None
    """Rules evaluated with a NOT logical operator.

    To match the policy, a user cannot meet any of the Exclude rules.
    """

    include: Optional[List[AccessRule]] = None
    """Rules evaluated with an OR logical operator.

    A user needs to meet only one of the Include rules.
    """

    isolation_required: Optional[bool] = None
    """
    Require this application to be served in an isolated browser for users matching
    this policy. 'Client Web Isolation' must be on for the account in order to use
    this feature.
    """

    mfa_config: Optional[MfaConfig] = None
    """Configures multi-factor authentication (MFA) settings."""

    name: Optional[str] = None
    """The name of the Access policy."""

    precedence: Optional[int] = None
    """The order of execution for this policy.

    Must be unique for each policy within an app.
    """

    purpose_justification_prompt: Optional[str] = None
    """A custom message that will appear on the purpose justification screen."""

    purpose_justification_required: Optional[bool] = None
    """Require users to enter a justification when they log in to the application."""

    require: Optional[List[AccessRule]] = None
    """Rules evaluated with an AND logical operator.

    To match the policy, a user must meet all of the Require rules.
    """

    session_duration: Optional[str] = None
    """The amount of time that tokens issued for the application will be valid.

    Must be in the format `300ms` or `2h45m`. Valid time units are: ns, us (or µs),
    ms, s, m, h.
    """

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .product import Product as Product
from .lockdown import Lockdown as Lockdown
from .lockdown_url import LockdownURL as LockdownURL
from .configuration import Configuration as Configuration
from .firewall_rule import FirewallRule as FirewallRule
from .deleted_filter import DeletedFilter as DeletedFilter
from .rule_list_params import RuleListParams as RuleListParams
from .asn_configuration import ASNConfiguration as ASNConfiguration
from .ipv6_configuration import IPV6Configuration as IPV6Configuration
from .rule_create_params import RuleCreateParams as RuleCreateParams
from .rule_update_params import RuleUpdateParams as RuleUpdateParams
from .configuration_param import ConfigurationParam as ConfigurationParam
from .ua_rule_list_params import UARuleListParams as UARuleListParams
from .lockdown_list_params import LockdownListParams as LockdownListParams
from .ua_rule_get_response import UARuleGetResponse as UARuleGetResponse
from .country_configuration import CountryConfiguration as CountryConfiguration
from .rule_bulk_edit_params import RuleBulkEditParams as RuleBulkEditParams
from .ua_rule_create_params import UARuleCreateParams as UARuleCreateParams
from .ua_rule_list_response import UARuleListResponse as UARuleListResponse
from .ua_rule_update_params import UARuleUpdateParams as UARuleUpdateParams
from .lockdown_create_params import LockdownCreateParams as LockdownCreateParams
from .lockdown_update_params import LockdownUpdateParams as LockdownUpdateParams
from .access_rule_edit_params import AccessRuleEditParams as AccessRuleEditParams
from .access_rule_list_params import AccessRuleListParams as AccessRuleListParams
from .asn_configuration_param import ASNConfigurationParam as ASNConfigurationParam
from .rule_bulk_update_params import RuleBulkUpdateParams as RuleBulkUpdateParams
from .ua_rule_create_response import UARuleCreateResponse as UARuleCreateResponse
from .ua_rule_delete_response import UARuleDeleteResponse as UARuleDeleteResponse
from .ua_rule_update_response import UARuleUpdateResponse as UARuleUpdateResponse
from .access_rule_get_response import AccessRuleGetResponse as AccessRuleGetResponse
from .ipv6_configuration_param import IPV6ConfigurationParam as IPV6ConfigurationParam
from .lockdown_delete_response import LockdownDeleteResponse as LockdownDeleteResponse
from .access_rule_create_params import AccessRuleCreateParams as AccessRuleCreateParams
from .access_rule_edit_response import AccessRuleEditResponse as AccessRuleEditResponse
from .access_rule_list_response import AccessRuleListResponse as AccessRuleListResponse
from .lockdown_ip_configuration import LockdownIPConfiguration as LockdownIPConfiguration
from .access_rule_create_response import AccessRuleCreateResponse as AccessRuleCreateResponse
from .access_rule_delete_response import AccessRuleDeleteResponse as AccessRuleDeleteResponse
from .country_configuration_param import CountryConfigurationParam as CountryConfigurationParam
from .lockdown_cidr_configuration import LockdownCIDRConfiguration as LockdownCIDRConfiguration
from .access_rule_ip_configuration import AccessRuleIPConfiguration as AccessRuleIPConfiguration
from .access_rule_cidr_configuration import AccessRuleCIDRConfiguration as AccessRuleCIDRConfiguration
from .lockdown_ip_configuration_param import LockdownIPConfigurationParam as LockdownIPConfigurationParam
from .lockdown_cidr_configuration_param import LockdownCIDRConfigurationParam as LockdownCIDRConfigurationParam
from .access_rule_ip_configuration_param import AccessRuleIPConfigurationParam as AccessRuleIPConfigurationParam
from .access_rule_cidr_configuration_param import AccessRuleCIDRConfigurationParam as AccessRuleCIDRConfigurationParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .identity_provider_type import IdentityProviderType
from .identity_provider_scim_config import IdentityProviderSCIMConfig

__all__ = ["AzureAD", "Config", "SAMLCertificateSet", "SAMLCertificateSetCurrentCertificate"]


class Config(BaseModel):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    claims: Optional[List[str]] = None
    """Custom claims"""

    client_id: Optional[str] = None
    """Your OAuth Client ID"""

    client_secret: Optional[str] = None
    """Your OAuth Client Secret"""

    conditional_access_enabled: Optional[bool] = None
    """Should Cloudflare try to load authentication contexts from your account"""

    directory_id: Optional[str] = None
    """Your Azure directory uuid"""

    email_claim_name: Optional[str] = None
    """The claim name for email in the id_token response."""

    prompt: Optional[Literal["login", "select_account", "none"]] = None
    """Indicates the type of user interaction that is required.

    prompt=login forces the user to enter their credentials on that request,
    negating single-sign on. prompt=none is the opposite. It ensures that the user
    isn't presented with any interactive prompt. If the request can't be completed
    silently by using single-sign on, the Microsoft identity platform returns an
    interaction_required error. prompt=select_account interrupts single sign-on
    providing account selection experience listing all the accounts either in
    session or any remembered account or an option to choose to use a different
    account altogether.
    """

    support_groups: Optional[bool] = None
    """Should Cloudflare try to load groups from your account"""


class SAMLCertificateSetCurrentCertificate(BaseModel):
    """The currently active certificate used for encrypting SAML assertions"""

    is_current: bool
    """Indicates whether this is the currently active certificate"""

    not_after: datetime
    """Certificate expiration date.

    Certificates are automatically rotated 30 days before expiration.
    """

    public_certificate: str
    """
    PEM-encoded X.509 certificate containing the public key. Configure this
    certificate in your external SAML Identity Provider to enable encryption.
    """

    uid: str
    """Unique identifier for the certificate"""


class SAMLCertificateSet(BaseModel):
    """
    The SAML encryption certificate set details, including current and previous certificates.
    Only present for SAML identity providers with a certificate set assigned.
    """

    created_at: datetime
    """Timestamp when the certificate set was created"""

    uid: str
    """Unique identifier for the certificate set"""

    updated_at: datetime
    """Timestamp when the certificate set was last updated (e.g., during rotation)"""

    current_certificate: Optional[SAMLCertificateSetCurrentCertificate] = None
    """The currently active certificate used for encrypting SAML assertions"""

    previous_certificate: Optional[object] = None
    """The previous certificate, maintained during rotation to ensure continuity.

    Null if no rotation has occurred. Mirrors the structure of `saml_certificate`.
    """


class AzureAD(BaseModel):
    config: Config
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: str
    """The name of the identity provider, shown to users on the login page."""

    type: IdentityProviderType
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    id: Optional[str] = None
    """UUID."""

    saml_certificate_set: Optional[SAMLCertificateSet] = None
    """
    The SAML encryption certificate set details, including current and previous
    certificates. Only present for SAML identity providers with a certificate set
    assigned.
    """

    saml_certificate_set_id: Optional[str] = None
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: Optional[IdentityProviderSCIMConfig] = None
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SettingUpdateParams"]


class SettingUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    disable_for_time: float
    """
    Sets the time limit, in seconds, that a user can use an override code to bypass
    WARP.
    """

    external_emergency_signal_enabled: bool
    """Controls whether the external emergency disconnect feature is enabled."""

    external_emergency_signal_fingerprint: str
    """
    The SHA256 fingerprint (64 hexadecimal characters) of the HTTPS server
    certificate for the external_emergency_signal_url. If provided, the WARP client
    will use this value to verify the server's identity. The device will ignore any
    response if the server's certificate fingerprint does not exactly match this
    value.
    """

    external_emergency_signal_interval: str
    """
    The interval at which the WARP client fetches the emergency disconnect signal,
    formatted as a duration string (e.g., "5m", "2m30s", "1h"). Minimum 30 seconds.
    """

    external_emergency_signal_url: str
    """The HTTPS URL from which to fetch the emergency disconnect signal.

    Must use HTTPS and have an IPv4 or IPv6 address as the host.
    """

    gateway_proxy_enabled: bool
    """Enable gateway proxy filtering on TCP."""

    gateway_udp_proxy_enabled: bool
    """Enable gateway proxy filtering on UDP."""

    root_certificate_installation_enabled: bool
    """Enable installation of cloudflare managed root certificate."""

    use_zt_virtual_ip: bool
    """Enable using CGNAT virtual IPv4."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scim import (
    SCIMResource,
    AsyncSCIMResource,
    SCIMResourceWithRawResponse,
    AsyncSCIMResourceWithRawResponse,
    SCIMResourceWithStreamingResponse,
    AsyncSCIMResourceWithStreamingResponse,
)
from .saml_certificate import (
    SAMLCertificateResource,
    AsyncSAMLCertificateResource,
    SAMLCertificateResourceWithRawResponse,
    AsyncSAMLCertificateResourceWithRawResponse,
    SAMLCertificateResourceWithStreamingResponse,
    AsyncSAMLCertificateResourceWithStreamingResponse,
)
from .identity_providers import (
    IdentityProvidersResource,
    AsyncIdentityProvidersResource,
    IdentityProvidersResourceWithRawResponse,
    AsyncIdentityProvidersResourceWithRawResponse,
    IdentityProvidersResourceWithStreamingResponse,
    AsyncIdentityProvidersResourceWithStreamingResponse,
)

__all__ = [
    "SCIMResource",
    "AsyncSCIMResource",
    "SCIMResourceWithRawResponse",
    "AsyncSCIMResourceWithRawResponse",
    "SCIMResourceWithStreamingResponse",
    "AsyncSCIMResourceWithStreamingResponse",
    "SAMLCertificateResource",
    "AsyncSAMLCertificateResource",
    "SAMLCertificateResourceWithRawResponse",
    "AsyncSAMLCertificateResourceWithRawResponse",
    "SAMLCertificateResourceWithStreamingResponse",
    "AsyncSAMLCertificateResourceWithStreamingResponse",
    "IdentityProvidersResource",
    "AsyncIdentityProvidersResource",
    "IdentityProvidersResourceWithRawResponse",
    "AsyncIdentityProvidersResourceWithRawResponse",
    "IdentityProvidersResourceWithStreamingResponse",
    "AsyncIdentityProvidersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["IPListRule", "IPList"]


class IPList(BaseModel):
    id: str
    """The ID of a previously created IP list."""


class IPListRule(BaseModel):
    """Matches an IP address from a list."""

    ip_list: IPList



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.devices import ip_profile_list_params, ip_profile_create_params, ip_profile_update_params
from ....types.zero_trust.devices.ip_profile import IPProfile
from ....types.zero_trust.devices.ip_profile_delete_response import IPProfileDeleteResponse

__all__ = ["IPProfilesResource", "AsyncIPProfilesResource"]


class IPProfilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IPProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IPProfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        match: str,
        name: str,
        precedence: int,
        subnet_id: str,
        description: Optional[str] | Omit = omit,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """Creates a WARP Device IP profile.

        Currently, only IPv4 Device subnets can be
        associated.

        Args:
          match:
              The wirefilter expression to match registrations. Available values:
              "identity.name", "identity.email", "identity.groups.id", "identity.groups.name",
              "identity.groups.email", "identity.saml_attributes".

          name: A user-friendly name for the Device IP profile.

          precedence: The precedence of the Device IP profile. Lower values indicate higher
              precedence. Device IP profile will be evaluated in ascending order of this
              field.

          subnet_id: The ID of the Subnet.

          description: An optional description of the Device IP profile.

          enabled: Whether the Device IP profile will be applied to matching devices.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/devices/ip-profiles", account_id=account_id),
            body=maybe_transform(
                {
                    "match": match,
                    "name": name,
                    "precedence": precedence,
                    "subnet_id": subnet_id,
                    "description": description,
                    "enabled": enabled,
                },
                ip_profile_create_params.IPProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )

    def update(
        self,
        profile_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        match: str | Omit = omit,
        name: str | Omit = omit,
        precedence: int | Omit = omit,
        subnet_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """Updates a WARP Device IP profile.

        Currently, only IPv4 Device subnets can be
        associated.

        Args:
          description: An optional description of the Device IP profile.

          enabled: Whether the Device IP profile is enabled.

          match:
              The wirefilter expression to match registrations. Available values:
              "identity.name", "identity.email", "identity.groups.id", "identity.groups.name",
              "identity.groups.email", "identity.saml_attributes".

          name: A user-friendly name for the Device IP profile.

          precedence: The precedence of the Device IP profile. Lower values indicate higher
              precedence. Device IP profile will be evaluated in ascending order of this
              field.

          subnet_id: The ID of the Subnet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "match": match,
                    "name": name,
                    "precedence": precedence,
                    "subnet_id": subnet_id,
                },
                ip_profile_update_params.IPProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )

    def list(
        self,
        *,
        account_id: str,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[IPProfile]:
        """
        Lists WARP Device IP profiles.

        Args:
          per_page: The number of IP profiles to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/ip-profiles", account_id=account_id),
            page=SyncSinglePage[IPProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"per_page": per_page}, ip_profile_list_params.IPProfileListParams),
            ),
            model=IPProfile,
        )

    def delete(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfileDeleteResponse:
        """
        Delete a WARP Device IP profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfileDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[IPProfileDeleteResponse], ResultWrapper[IPProfileDeleteResponse]),
        )

    def get(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """
        Fetches a single WARP Device IP profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )


class AsyncIPProfilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIPProfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        match: str,
        name: str,
        precedence: int,
        subnet_id: str,
        description: Optional[str] | Omit = omit,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """Creates a WARP Device IP profile.

        Currently, only IPv4 Device subnets can be
        associated.

        Args:
          match:
              The wirefilter expression to match registrations. Available values:
              "identity.name", "identity.email", "identity.groups.id", "identity.groups.name",
              "identity.groups.email", "identity.saml_attributes".

          name: A user-friendly name for the Device IP profile.

          precedence: The precedence of the Device IP profile. Lower values indicate higher
              precedence. Device IP profile will be evaluated in ascending order of this
              field.

          subnet_id: The ID of the Subnet.

          description: An optional description of the Device IP profile.

          enabled: Whether the Device IP profile will be applied to matching devices.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/devices/ip-profiles", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "match": match,
                    "name": name,
                    "precedence": precedence,
                    "subnet_id": subnet_id,
                    "description": description,
                    "enabled": enabled,
                },
                ip_profile_create_params.IPProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )

    async def update(
        self,
        profile_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        match: str | Omit = omit,
        name: str | Omit = omit,
        precedence: int | Omit = omit,
        subnet_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """Updates a WARP Device IP profile.

        Currently, only IPv4 Device subnets can be
        associated.

        Args:
          description: An optional description of the Device IP profile.

          enabled: Whether the Device IP profile is enabled.

          match:
              The wirefilter expression to match registrations. Available values:
              "identity.name", "identity.email", "identity.groups.id", "identity.groups.name",
              "identity.groups.email", "identity.saml_attributes".

          name: A user-friendly name for the Device IP profile.

          precedence: The precedence of the Device IP profile. Lower values indicate higher
              precedence. Device IP profile will be evaluated in ascending order of this
              field.

          subnet_id: The ID of the Subnet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "match": match,
                    "name": name,
                    "precedence": precedence,
                    "subnet_id": subnet_id,
                },
                ip_profile_update_params.IPProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )

    def list(
        self,
        *,
        account_id: str,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[IPProfile, AsyncSinglePage[IPProfile]]:
        """
        Lists WARP Device IP profiles.

        Args:
          per_page: The number of IP profiles to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/ip-profiles", account_id=account_id),
            page=AsyncSinglePage[IPProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"per_page": per_page}, ip_profile_list_params.IPProfileListParams),
            ),
            model=IPProfile,
        )

    async def delete(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfileDeleteResponse:
        """
        Delete a WARP Device IP profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfileDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[IPProfileDeleteResponse], ResultWrapper[IPProfileDeleteResponse]),
        )

    async def get(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPProfile:
        """
        Fetches a single WARP Device IP profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/devices/ip-profiles/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[IPProfile]._unwrapper,
            ),
            cast_to=cast(Type[IPProfile], ResultWrapper[IPProfile]),
        )


class IPProfilesResourceWithRawResponse:
    def __init__(self, ip_profiles: IPProfilesResource) -> None:
        self._ip_profiles = ip_profiles

        self.create = to_raw_response_wrapper(
            ip_profiles.create,
        )
        self.update = to_raw_response_wrapper(
            ip_profiles.update,
        )
        self.list = to_raw_response_wrapper(
            ip_profiles.list,
        )
        self.delete = to_raw_response_wrapper(
            ip_profiles.delete,
        )
        self.get = to_raw_response_wrapper(
            ip_profiles.get,
        )


class AsyncIPProfilesResourceWithRawResponse:
    def __init__(self, ip_profiles: AsyncIPProfilesResource) -> None:
        self._ip_profiles = ip_profiles

        self.create = async_to_raw_response_wrapper(
            ip_profiles.create,
        )
        self.update = async_to_raw_response_wrapper(
            ip_profiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            ip_profiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            ip_profiles.delete,
        )
        self.get = async_to_raw_response_wrapper(
            ip_profiles.get,
        )


class IPProfilesResourceWithStreamingResponse:
    def __init__(self, ip_profiles: IPProfilesResource) -> None:
        self._ip_profiles = ip_profiles

        self.create = to_streamed_response_wrapper(
            ip_profiles.create,
        )
        self.update = to_streamed_response_wrapper(
            ip_profiles.update,
        )
        self.list = to_streamed_response_wrapper(
            ip_profiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            ip_profiles.delete,
        )
        self.get = to_streamed_response_wrapper(
            ip_profiles.get,
        )


class AsyncIPProfilesResourceWithStreamingResponse:
    def __init__(self, ip_profiles: AsyncIPProfilesResource) -> None:
        self._ip_profiles = ip_profiles

        self.create = async_to_streamed_response_wrapper(
            ip_profiles.create,
        )
        self.update = async_to_streamed_response_wrapper(
            ip_profiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ip_profiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ip_profiles.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            ip_profiles.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SiteInfoUpdateParams"]


class SiteInfoUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    auto_install: bool
    """
    If enabled, the JavaScript snippet is automatically injected for orange-clouded
    sites.
    """

    enabled: bool
    """Enables or disables RUM.

    This option can be used only when auto_install is set to true.
    """

    host: str
    """The hostname to use for gray-clouded sites."""

    lite: bool
    """
    If enabled, the JavaScript snippet will not be injected for visitors from the
    EU.
    """

    zone_tag: str
    """The zone identifier."""
